/*
Copyright 2003/2004, Freie Universitaet Berlin. All rights reserved.

These sources were developed at the Freie Universitt Berlin, Computer
Systems and Telematics group.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

- Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.
 
- Neither the name of Freie Universitaet Berlin (FUB) nor the names of its
contributors may be used to endorse or promote products derived from
this software without specific prior written permission.

This software is provided by FUB and the contributors on an "as is"
basis, without any representations or warranties of any kind, express
or implied including, but not limited to, representations or
warranties of non-infringement, merchantability or fitness for a
particular purpose. In no event shall FUB or contributors be liable
for any direct, indirect, incidental, special, exemplary, or
consequential damages (including, but not limited to, procurement of
substitute goods or services; loss of use, data, or profits; or
business interruption) however caused and on any theory of liability,
whether in contract, strict liability, or tort (including negligence
or otherwise) arising in any way out of the use of this software, even
if advised of the possibility of such damage.

This implementation was developed by the CST group at the FUB.
Contributors: Bjoern Lichtblau

For documentation and questions please use the web site
http://www.scatterweb.net and the mailinglist
scatterweb@lists.spline.inf.fu-berlin.de (subscription via the Website).
Berlin, 2003/2004/2005
*/
 
#include "ScatterWeb.System.h"

UINT8 sensorFlags;

#if defined(ESB430) || defined(EYE430) || defined(ECR430)

#define SENSORCALLBACK(Sensor, State)    do { \
                                             sdata_t data; \
                                             data.id = Configuration.id; \
                                             data.sensor = (Sensor); \
                                             data.timeStamp = CURRENT_TIME; \
                                             data.value = State; \
                                             if(callbacks[C_SENSOR]) callbacks[C_SENSOR](&data); }while(0) 

UINT16 _vibDebounceDelay;
UINT16 _tmpInterval;   ///< How often is the temperature read in systems ticks (~1ms)
UINT16 _tmpStartRange; ///< How much must the temperature differs from the last value.
UINT16 _tmpDiffStep;   ///< How much measures until Range is decremented with Diff.
UINT16 _tmpDiff;       ///< How much the Range is decreased every DiffStep measures.
temp_t _tmpLast;       ///< Last signaled temperature.
UINT16 _tmpSteps;      ///< Current steps, if this reaches DiffStep do Range -= Diff 
UINT16 _tmpRange;      ///< Current range, if abs(current temperature - last temperature) > range signal new temperature.
UINT16 _batInterval;   ///< How often is the battery voltage read in systems ticks (~1ms).
UINT16 _batThresh;     ///< When battery voltage drops below this level, every _batInterval ticks voltage will get signaled.

void vibCheck() {
  if(P1IFG & SENSOR_VIBRATION) {
      P1IFG &= ~SENSOR_VIBRATION;
      Timers_add(_vibDebounceDelay, vibCheck, 0xFFFF);    
  }
  else {
    P1IE |= SENSOR_VIBRATION;
  	SENSORCALLBACK(SENSOR_VIBRATION, 0);
  }
}

void temperatureCheck() {
  temp_t temperature;
  sdata_t data = { Configuration.id, SENSOR_TEMPERATURE, CURRENT_TIME, 0 };
  UINT16 diff;
  #if defined(ESB430) || defined(EYE430)
  HAL_clStartConversion();
  #endif
  temperature = HAL_getTemperature(); // Adjust range if enabled.
  if(_tmpDiffStep>0) {
    _tmpSteps++;
    if(_tmpSteps==_tmpDiffStep) {
      _tmpSteps = 0;
      if(_tmpRange < _tmpDiff) _tmpRange = 0;
      else                     _tmpRange -= _tmpDiff;
    }
  }
  if(_tmpLast.s < temperature.s) diff = temperature.s - _tmpLast.s;
  else                           diff = _tmpLast.s - temperature.s;
  if(diff >= _tmpRange) {
    data.value = temperature.u;
    if(callbacks[C_SENSOR]) callbacks[C_SENSOR](&data); 
    _tmpLast = temperature;
    _tmpRange = _tmpStartRange;
    _tmpSteps = 0;
  }
  Timers_add(_tmpInterval, temperatureCheck, 0xFFFF);
}

void batteryCheck() {
  sdata_t data = { Configuration.id, SENSOR_VOLTAGE, CURRENT_TIME, 0 };
  unsigned int bat = BATTERYVOLTAGE;
  if(bat < _batThresh) {
    data.value = bat;
    if(callbacks[C_SENSOR]) callbacks[C_SENSOR](&data); 
  }
  Timers_add(_batInterval, batteryCheck, 0xFFFF);
}

interrupt(PORT1_VECTOR) p1_irq (void) {
  #if defined(ESB430) || defined(EYE430)
  if(P1IE & P1IFG & SENSOR_MOVEMENT) {
    sensorFlags |= SENSOR_MOVEMENT;
    P1IE  &= ~SENSOR_MOVEMENT;
    P1IFG &= ~SENSOR_MOVEMENT;
  }
  if(P1IE & P1IFG & 0x04) {
    if (libregistry[LIB_RC5_EVE]) libregistry[LIB_RC5_EVE]();
  }
  #endif
  if(P1IE & P1IFG & SENSOR_VIBRATION) {
    sensorFlags |= SENSOR_VIBRATION;
    P1IE  &= ~SENSOR_VIBRATION;
    P1IFG &= ~SENSOR_VIBRATION;
  } 
  WAKEUPLPM1(p1_irq, MF_SENSORS);
}

#endif

#if defined(ESB430) || defined(EYE430)

UINT16 _pirDebounceDelay;
UINT16 _buttonDebounceDelay;
UINT16 _micDiff;
UINT16 _micDebounceDelay;

void pirCheck() {
  if(P1IFG & SENSOR_MOVEMENT){
      P1IFG &= ~SENSOR_MOVEMENT;
      Timers_add(_pirDebounceDelay, pirCheck, 0xFFFF);    
  }
  else {
    P1IE |= SENSOR_MOVEMENT;
  	SENSORCALLBACK(SENSOR_MOVEMENT, 0);
  }
}

void buttonCheck() {
  if(P2IES & SENSOR_BUTTON) {     // button down was debounced
    if(!(P2IN & SENSOR_BUTTON)) { // button is still down
      P2IES &= ~SENSOR_BUTTON;    // set interrupt to detect rising edge (button up)
      P2IFG &= ~SENSOR_BUTTON;    // clear ifg
      P2IE |= SENSOR_BUTTON;      // and enable interrupt again
    }
    else {                        // button already up again
  	    SENSORCALLBACK(SENSOR_BUTTON, 0);
       	Timers_add(_buttonDebounceDelay, buttonCheck, 0xFFFF);
         P2IES &= ~SENSOR_BUTTON; // debounce is now for button up        	
    }
  }
  else {                          // button up was debounced
    if(P2IN & SENSOR_BUTTON) {    //button is still up
      P2IES |= SENSOR_BUTTON;     // set interrupt to detect falling edge (button down)
      P2IFG &= ~SENSOR_BUTTON;    // clear ifg
      P2IE |= SENSOR_BUTTON;      // and enable interrupt again
    }
    else {                        // button already down again
   	  SENSORCALLBACK(SENSOR_BUTTON, 1);
      Timers_add(_buttonDebounceDelay, buttonCheck, 0xFFFF);             	        
      P2IES |= SENSOR_BUTTON;     // debounce is now for button down

    }
  }
}

void micCheck2() {
  if(MIC & MICIFG) {
      MIC = 0;
      Timers_add(_micDebounceDelay, micCheck2, 0xFFFF);
  }
  else {
    MIC = MICIE;
  	SENSORCALLBACK(SENSOR_MICROPHONE, 0);     
  }
}

void micCheck1() {
  if(MIC & MICIFG) {  // still loud
      MIC = 0;
      Timers_add(_micDebounceDelay, micCheck2, 0xFFFF);
      SENSORCALLBACK(SENSOR_MICROPHONE, 1);
  }
  else {              // silent again
    MIC = MICIE;    
  }
}

#endif

#if defined(ESB430) || defined(EYE430)
interrupt(PORT2_VECTOR) p2_irq (void) { /// ISR for Port 2, only detects button presses.
  if(P2IE & P2IFG & SENSOR_BUTTON){
    sensorFlags |= SENSOR_BUTTON;
    P2IE  &= ~SENSOR_BUTTON;
    P2IFG &= ~SENSOR_BUTTON;
  } 
  WAKEUPLPM1(p1_irq, MF_SENSORS);
}
#endif

bool Data_vibrationOn(UINT16 delay) {
  #if defined(ESB430) || defined(EYE430) || defined(ECR430)
  _vibDebounceDelay = delay;
  P1IFG &= ~SENSOR_VIBRATION;
  P1IE |= SENSOR_VIBRATION;
  return True;
  #else
  return False;
  #endif
}

bool Data_vibrationOff() {
  #if defined(ESB430) || defined(EYE430) || defined(ECR430)
  P1IE &= ~SENSOR_VIBRATION;
  Timers_remove(vibCheck, 0xFFFF);
  sensorFlags &= ~ SENSOR_VIBRATION;
  return True;
  #else
  return False;
  #endif
}

bool Data_movementOn(UINT16 delay) {
  #if defined(ESB430) || defined(EYE430)
  _pirDebounceDelay = delay;
  P1IFG &= ~SENSOR_MOVEMENT;
  P1IE |= SENSOR_MOVEMENT;
  return True;
  #else
  return False;
  #endif
}

bool Data_movementOff(){
  #if defined(ESB430) || defined(EYE430)
  P1IE &= ~SENSOR_MOVEMENT;
  Timers_remove(pirCheck, 0xFFFF);
  sensorFlags &= ~ SENSOR_MOVEMENT;
  return True;
  #else
  return False;
  #endif
}

bool Data_buttonOn(UINT16 delay) {
  #if defined(ESB430) || defined(EYE430)
  _buttonDebounceDelay = delay;
  P2IES |= SENSOR_BUTTON;         // wait for raising edge
  P2IFG &= ~SENSOR_BUTTON;
  P2IE |= SENSOR_BUTTON;
  return True;
  #else
  return False;
  #endif
}

bool Data_buttonOff() {
  #if defined(ESB430) || defined(EYE430)
  P1IE &= ~SENSOR_BUTTON;
  Timers_remove(buttonCheck, 0xFFFF);
  sensorFlags &= ~ SENSOR_BUTTON;
  return True;
  #else
  return False;
  #endif
}

bool Data_microphoneOn(UINT16 delay, UINT16 diff) {
  #if defined(ESB430) || defined(EYE430)
  _micDiff = diff;
  _micDebounceDelay = delay;
  MIC &= ~MICIFG;
  MIC |= MICIE;
  return True;
  #else
  return False;
  #endif
}

bool Data_microphoneOff(){
  #if defined(ESB430) || defined(EYE430)
  MIC = 0x00;
  Timers_remove(micCheck1, 0xFFFF);
  Timers_remove(micCheck2, 0xFFFF);
  sensorFlags &= ~SENSOR_MICROPHONE;
  return True;
  #else
  return False;
  #endif
}

bool Data_temperatureOn(UINT16 interval, UINT16 range, UINT16 diff, UINT16 stepsperdiff) {
  #if defined(ESB430) || defined(EYE430) || defined(ECR430)
  #if defined(ESB430) || defined(EYE430)
  HAL_clStartConversion();
  #endif
  _tmpInterval = interval;
  _tmpStartRange = range;
  _tmpDiff = diff;
  _tmpDiffStep = stepsperdiff;
  _tmpLast.u = 0xC000;
  _tmpRange = _tmpStartRange;
  _tmpSteps = 0;
  Timers_add(_tmpInterval, temperatureCheck, 0xFFFF);
  return True;
  #else
  return False;
  #endif
}

bool Data_temperatureOff() {
  #if defined(ESB430) || defined(EYE430) || defined(ECR430)
  Timers_remove(temperatureCheck, 0xFFFF);
  return True;
  #else
  return False;
  #endif
}

bool Data_batteryOn(UINT16 interval, UINT16 thresh) {
  #if defined(ESB430) || defined(EYE430) || defined(ECR430)
  _batInterval = interval;
  _batThresh = thresh;
  Timers_add(_batInterval, batteryCheck, 0xFFFF);
  return True;
  #else
  return False;
  #endif  
}

bool Data_batteryOff() {
  #if defined(ESB430) || defined(EYE430) || defined(ECR430)
  Timers_remove(batteryCheck, 0xFFFF);
  return True;
  #else
  return False;
  #endif
}

bool Data_init() {
  #if defined(ESB430) || defined(EYE430) || defined(ECR430)
  sensorFlags = 0x00;
  callbacks[C_SENSOR] = NULL;
  Data_on();
  return True;
  #else
  Data_on();
  return False;
  #endif
}

bool Data_on() {
  #if defined(ESB430) || defined(EYE430) || defined(ECR430)
  #if defined(ECR430)
  P2OUT |= 0x08;
  System_wait(2000);
  P5OUT |= 0x10;
  #endif
  P5OUT &= ~0x20; // turn power on for sensors
  #endif
  #if defined(ESB430) || defined(EYE430)
  System_wait(150000); // Needed for microphone
  return True;
  #else
  return False;
  #endif  
}

bool Data_off() {
  #if defined(ESB430) || defined(EYE430) || defined(ECR430)
  P5OUT |= 0x20;  // turn power off for sensors
  #if defined(ECR430)
  // Important for ECR430: When disabling Power (set P55 to 1) set
  // P50-P54 and P23 to ouput 0, else current will flow from them.
  P2OUT &= ~0x08;
  P5OUT &= ~0x1C;
  #endif
  return True;
  #else
  return False;
  #endif
}

bool Data_sensorHandler() {
  #if defined(ESB430) || defined(EYE430) || defined(ECR430)
  if(sensorFlags & SENSOR_VIBRATION) {
      Timers_add(_vibDebounceDelay, vibCheck, 0xFFFF);  
      SENSORCALLBACK(SENSOR_VIBRATION, 1);
      sensorFlags &= ~SENSOR_VIBRATION;    
  }
  #if defined(ESB430) || defined(EYE430)
  if(sensorFlags & SENSOR_MOVEMENT) {
      Timers_add(_pirDebounceDelay, pirCheck, 0xFFFF);  
      SENSORCALLBACK(SENSOR_MOVEMENT, 1);
      sensorFlags &= ~SENSOR_MOVEMENT;    
  }
  if(sensorFlags & SENSOR_BUTTON) {
      Timers_add(_buttonDebounceDelay, buttonCheck, 0xFFFF);  
    	if((P2IES & SENSOR_BUTTON))  SENSORCALLBACK(SENSOR_BUTTON, 1);
    	if(!(P2IES & SENSOR_BUTTON)) SENSORCALLBACK(SENSOR_BUTTON, 0);   	
      sensorFlags &= ~SENSOR_BUTTON;    
  }
  if(sensorFlags & SENSOR_MICROPHONE) {
      sensorFlags &= ~SENSOR_MICROPHONE;    
      MIC = 0;
      Timers_add(50, micCheck1, 0xFFFF);
  }
  #endif
  // Reset flag if nothing more needs handling
  dint(); nop();
  if(sensorFlags==0) runModule &= ~MF_SENSORS;
  eint();
  return True;
  #else
  return False;
  #endif
}

bool Data_beeperOn() {
  #if defined(ESB430) || defined(EYE430) || defined(ESS430)
  P2OUT |=  0x08;
  return True;
  #else
  return False;
  #endif
}


bool Data_beeperOff() {
  #if defined(ESB430) || defined(EYE430) || defined(ESS430)
  P2OUT &= ~0x08;
  return True;
  #else
  return False;
  #endif
}

bool Data_beeperToggle() {
  #if defined(ESB430) || defined(EYE430) || defined(ESS430)
  P2OUT ^=  0x08;
  return True;
  #else
  return False;
  #endif
}

bool Data_greenOn() {
  #if defined(ESB430) || defined(EYE430) || defined(ECR430) || defined(ESS430)
  do{ P2OUT &= ~0x02; }while(0);
  return True;
  #else
  return False;
  #endif
}

bool Data_greenOff() {
  #if defined(ESB430) || defined(EYE430) || defined(ECR430) || defined(ESS430)
  do{ P2OUT |=  0x02; }while(0);
  return True;
  #else
  return False;
  #endif
}

bool Data_greenToggle() {
  #if defined(ESB430) || defined(EYE430) || defined(ECR430) || defined(ESS430)
  do{ P2OUT ^=  0x02; }while(0);
  return True;
  #else
  return False;
  #endif
}

bool Data_redOn() {
  #if defined(ESB430) || defined(EYE430) || defined(ECR430) || defined(ESS430)
  do{ P2OUT &= ~0x01; }while(0);
  return True;
  #elif defined(ESF430)
  do{ P5OUT &= ~0x01; }while(0);
  return True;
  #else
  return False;
  #endif
}

bool Data_redOff() {
  #if defined(ESB430) || defined(EYE430) || defined(ECR430) || defined(ESS430)
  do{ P2OUT |=  0x01; }while(0);
  return True;
  #elif defined(ESF430)
  do{ P5OUT |=  0x01; }while(0);
  return True;
  #else
  return False;
  #endif
}

bool Data_redToggle() {
  #if defined(ESB430) || defined(EYE430) || defined(ECR430) || defined(ESS430)
  do{ P2OUT ^=  0x01; }while(0);
  return True;
  #elif defined(ESF430)
  do{ P5OUT ^=  0x01; }while(0);
  return True;
  #else
  return False;
  #endif
}

bool Data_yellowOn() {
  #if defined(ESB430) || defined(EYE430) || defined(ECR430) || defined(ESS430)
  do{ P2OUT &= ~0x04; }while(0);
  return True;
  #else
  return False;
  #endif
}

bool Data_yellowOff() {
  #if defined(ESB430) || defined(EYE430) || defined(ECR430) || defined(ESS430)
  do{ P2OUT |=  0x04; }while(0);
  return True;
  #else
  return False;
  #endif
}

bool Data_yellowToggle() {
  #if defined(ESB430) || defined(EYE430) || defined(ECR430) || defined(ESS430)
  do{ P2OUT ^=  0x04; }while(0);
  return True;
  #else
  return False;
  #endif
}
