/*
Copyright 2003/2004, Freie Universitaet Berlin. All rights reserved.

These sources were developed at the Freie Universitt Berlin, Computer
Systems and Telematics group.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

- Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.
 
- Neither the name of Freie Universitaet Berlin (FUB) nor the names of its
contributors may be used to endorse or promote products derived from
this software without specific prior written permission.

This software is provided by FUB and the contributors on an "as is"
basis, without any representations or warranties of any kind, express
or implied including, but not limited to, representations or
warranties of non-infringement, merchantability or fitness for a
particular purpose. In no event shall FUB or contributors be liable
for any direct, indirect, incidental, special, exemplary, or
consequential damages (including, but not limited to, procurement of
substitute goods or services; loss of use, data, or profits; or
business interruption) however caused and on any theory of liability,
whether in contract, strict liability, or tort (including negligence
or otherwise) arising in any way out of the use of this software, even
if advised of the possibility of such damage.

This implementation was developed by the CST group at the FUB.
Contributors: Marco Hutta

For documentation and questions please use the web site
http://www.scatterweb.net and the mailinglist
scatterweb@lists.spline.inf.fu-berlin.de (subscription via the Website).
Berlin, 2003/2004/2005
*/

#ifndef System_HAL
#define System_HAL

#if defined(ESB430) || defined(EYE430)
#define MICIFG    (0x01)
#define MICIE     (0x02)
#define MICVOLTAGE        (ADC12MEM2)
#define EXTERNVOLTAGE     (ADC12MEM3)
#define BATTERYVOLTAGE    (ADC12MEM4)
#define RXPVOLTAGE        (ADC12MEM5)
extern volatile UINT8 MIC; ///< 0x01 is like IFG, 0x02 like IE and will set sensorFlags.MIC
#endif

#if defined(ECR430)
#define BATTERYVOLTAGE    (ADC12MEM4)
#endif

#define STARTUP_POR       (0) ///< Power on or reset button.
#define STARTUP_SWRESET   (1) ///< call via reset().
#define STARTUP_REPROGRAM (2) ///< reprogramming
#define STARTUP_PANIC     (3) ///< call via wd_panic(). All states <=STARTUP_PANIC are considered OK!
#define STARTUP_WATCHDOG  (4) ///< Reason for reset was watchdog expiry.
#define STARTUP_KEYV      (5) ///< Flash control register key violation.
#define STARTUP_UNKNOWN   (6) ///< Unknown type of reset.
#define WDS_RUNNING       (0xAB01) ///< System at startup.
#define WDS_SWRESET       (0xAB02) ///< Software sets this shortly before writing bad value into FCTL3 to recognize a wanted reset.
#define WDS_REPROGRAM     (0xAB03)
#define WDS_PANIC         (0xAB04)
// Last updated RXPower AD value.
extern volatile UINT16 rxpTemp;
// RXPower AD value for last received packet, cleared by ::rxClear().
extern volatile UINT16 rxpValue;

/*
 * Function to start conversion
 *  
 * @return bool   True always
 *  
 */
bool HAL_clStartConversion();

/*
 * Function to get temperature
 *  
 * @return temp_t  current temperature
 *  
 */
temp_t HAL_getTemperature();

/*
 * Function to init ports
 *  
 * @return bool   True always
 *  
 */
bool HAL_initPorts();

/*
 * Function to set DCO
 *  
 * @return bool   True always
 *  
 */
bool HAL_setDCO();

/*
 * Function to start watchdog
 *  
 * @return bool   True always
 *  
 */
bool HAL_startWatchdog();

/*
 * Function to stop watchdog
 *  
 * @return bool   True always
 *  
 */
bool HAL_stopWatchdog();

/*
 * Function to check green Led
 *  
 * @return UINT8  state of green Led
 *  
 */
UINT8 HAL_greenLed();

/*
 * Function to check red Led
 *  
 * @return UINT8  state of red Led
 *  
 */
UINT8 HAL_redLed();

/*
 * Function to check yellow Led
 *  
 * @return UINT8  state of yellow Led
 *  
 */
UINT8 HAL_yellowLed();

/*
 * Function to init hardware
 *  
 * @return bool   True always
 *  
 */
bool HAL_initHardware();

#endif
