/*
Copyright 2003/2004, Freie Universitaet Berlin. All rights reserved.

These sources were developed at the Freie Universitt Berlin, Computer
Systems and Telematics group.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

- Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.
 
- Neither the name of Freie Universitaet Berlin (FUB) nor the names of its
contributors may be used to endorse or promote products derived from
this software without specific prior written permission.

This software is provided by FUB and the contributors on an "as is"
basis, without any representations or warranties of any kind, express
or implied including, but not limited to, representations or
warranties of non-infringement, merchantability or fitness for a
particular purpose. In no event shall FUB or contributors be liable
for any direct, indirect, incidental, special, exemplary, or
consequential damages (including, but not limited to, procurement of
substitute goods or services; loss of use, data, or profits; or
business interruption) however caused and on any theory of liability,
whether in contract, strict liability, or tort (including negligence
or otherwise) arising in any way out of the use of this software, even
if advised of the possibility of such damage.

This implementation was developed by the CST group at the FUB.
Contributors: Thomas Pietsch, Bjoern Lichtblau

For documentation and questions please use the web site
http://www.scatterweb.net and the mailinglist
scatterweb@lists.spline.inf.fu-berlin.de (subscription via the Website).
Berlin, 2003/2004/2005
*/



#include "ScatterWeb.System.h" 
#include <io.h>
#include <signal.h>
#include "ScatterWeb.RC5.h"

#if defined(ESB430) || defined(EYE430)
#define IS_IR_RECEIVING   (TBCCTL0 & CCIE)
#define RC5_A_TV1     0
#define RC5_A_TV2     1
#define RC5_A_VCR1    5
#define RC5_A_VCR2    6
#define RC5_C_0       0
#define RC5_C_1       1
#define RC5_C_2       2
#define RC5_C_3       3
#define RC5_C_4       4
#define RC5_C_5       5
#define RC5_C_6       6
#define RC5_C_7       7
#define RC5_C_8       8
#define RC5_C_9       9
#define RC5_C_REW     13
#define RC5_C_VU      16
#define RC5_C_VD      17
#define RC5_CODE    (ir_data & 0x003F)           ///< Returns code of received packet.
#define RC5_ADR     ((ir_data & 0x07C0) >> 6)    ///< Returns data of received packet.
#define RC5_TOG     ((ir_data & 0x0800) >> 11)   ///< Returns toggle bit of received packet.
#define RC5_ERR     ((ir_data & 0x2000) >> 13)   ///< If 1 error occured during transmission.
#define SIR1  (P1OUT |= 0x01) ///< MACRO: Puts IR sending diode high.
#define SIR0  (P1OUT &= 0xFE) ///< MACRO: Puts IR sending diode low.
#define SET_EBIT    (ir_data |= 0x2000)
#define CLEAR_EBIT (ir_data &= 0xDFFF)
#define SET_DABIT   (ir_data |= 0x1000)
#define CLEAR_DABIT (ir_data &= 0xEFFF)

#if defined(DCO_2458KHZ)
#define BIT75 3282      ///< 3 quarters of a bit after start, 3282 cyc @ 2,4576Mhz = 1335us.
#define BIT50 2188      ///< Half of bit length, 2188 cyc @ 2,4576Mhz = 890 us.
#elif defined(DCO_4506KHZ)
#define BIT75 6015      ///< 3 quarters of a bit after start, 6015 cyc @ 4,5056Mhz = 1335us.
#define BIT50 4009      ///< Half of bit length, 4009 cyc @ 4,5056Mhz = 890 us.
#endif

UINT16 ir_pos;             ///< current position in frame
UINT16 ir_data;            ///< here a received packet is saved
UINT16 ir_databuffer;      ///< temporary buffer for receiving
UINT8 ir_temp;           ///< saves the first half of the manchester bit
#endif

#if defined(ESB430) || defined(EYE430)
volatile void sendIRHigh(UINT16 j){
    volatile UINT16 i;
    for(i=0; i<j; i++){ // 26 us
      SIR1; SIR1; SIR1; SIR1; SIR1; SIR1; SIR1; // added 3
      SIR0; SIR0; SIR0; SIR0; 
      SIR0; SIR0; SIR0; SIR0; 
      SIR0; SIR0; SIR0; SIR0; 
      SIR0; SIR0; SIR0; SIR0; 
      SIR0; SIR0; // 25 alltogether
    }  
}

volatile void sendIRLow(UINT16 j){
    /* waelchli: int -> UINT16 in order to avoid warning */
    volatile UINT16 i;
    for(i=0; i<j; i++){
      SIR0; SIR0; SIR0; SIR0; SIR0; SIR0; SIR0; 
      SIR0; SIR0; SIR0; SIR0; 
      SIR0; SIR0; SIR0; SIR0; 
      SIR0; SIR0; SIR0; SIR0; 
      SIR0; SIR0; SIR0; SIR0;  
      SIR0; SIR0; // 25 together
    }  
}

interrupt(TIMERB0_VECTOR) Timer_B0() {
    if(ir_pos <= 25) {
      if(ir_pos % 2) {                             // odd position
        if(ir_temp && !(P1IN & 0x04)) {           // 1 - 0 -->  write 1 
          ir_databuffer +=1;
          ir_databuffer = ir_databuffer << 1;  
        } 
        else if(!ir_temp && (P1IN & 0x04)) {      // 0 - 1 -->  write 0 
          ir_databuffer = ir_databuffer << 1;  
        } 
        else { 
          SET_EBIT;
          if(P1IN & 0x04) ir_databuffer += 1;
          ir_databuffer = ir_databuffer << 1;
        }
      }
      else {                                    // even position
        ir_temp = P1IN & 0x04;
      }
    }
    if(ir_pos==25) {                          // end reached
      ir_databuffer = ir_databuffer >> 1; 

      if(! RC5_ERR && ( (ir_databuffer & 0x0FFF) != (ir_data & 0x0FFF) ) ){
        ir_data = ir_databuffer;
        SET_DABIT;
        Data_yellowOn();
        WAKEUPLPM1(Timer_B0, MF_RC5);
      }   
    }
    if(ir_pos==27) {
      TBCCTL0 &= ~CCIE;  
      Data_greenOff();  
      Data_yellowOff();    
      // reenable interrupt for falling edge
      P1IFG &= ~(0x04);
      P1IE  |= 0x04;                    // enable interrupt for recir RC5
    }
  ir_pos++;
  TBCCR0 += BIT50;        // set new interrupt
  TBCCTL0 &= ~CCIFG;
}
#endif

COMMAND(sir, SERIALONLY) { 		                            
    #if defined(ESB430) || defined(EYE430)
    UINT8 high;
    UINT8 low;
    UINT16 data;
    high = String_fromHexToByte((char*) &str[4]);
    low = String_fromHexToByte((char*) &str[6]);
    data = high;
    data = (data << 8) + low;
    RC5_send(data);
    Comm_print("%x gesendet. \n", data); 
    #else
    String_write(term_reply, "NOT SUPPORTED: %s (wrong platform)", str);
    #endif
}

bool RC5_init() {
  #if defined(ESB430) || defined(EYE430)
  System_registerLib(LIB_RC5_REC, RC5_receiveHandler);
  System_registerLib(LIB_RC5_EVE, RC5_eventHandler);
  return True;
  #else
  return False;
  #endif
  return True;
}

bool RC5_send(int data) {
#if defined(ESB430) || defined(EYE430)
    volatile int mask = 0x2000;
    data |= 0xF000;
    dint();
    while(mask != 0){
	if(!(mask & data)){
      sendIRHigh(36); // 0,890 ms needed, (16+36*110) @ 4505600 = 0,882 ms
      sendIRLow(36);  
    } else {
      sendIRLow(36);
      sendIRHigh(36);
    }
    mask /= 2;
  }
  eint();
  return True;
  #else
  return False;
  #endif
}

bool RC5_enable() {
  #if defined(ESB430) || defined(EYE430)
  P1IES |= 0x04;  // Important for IR-RC5 receive to detect the first FALLING edge
  P1IFG &= ~0x04;
  P1IE  |= 0x04; 
  return True;
  #else
  return False;
  #endif
}

bool RC5_disable() {
  #if defined(ESB430) || defined(EYE430)
  dint(); nop();
  TBCCTL0 &= ~CCIE; 
  P1IE &= ~0x04;
  eint();
  return True;
  #else
  return False;
  #endif
}

void RC5_receiveHandler() {
  #if defined(ESB430) || defined(EYE430)
  sdata_t data = { Configuration.id, SENSOR_RC5, CURRENT_TIME, 0 };
  data.value = ir_data;
  if(callbacks[C_SENSOR]) callbacks[C_SENSOR](&data);
  runModule &= ~MF_RC5;
  #endif
}

void RC5_eventHandler() {
  #if defined(ESB430) || defined(EYE430)
  if(P1IN & 0x04) return; // high again, just a peak
  ir_pos = 0;
  ir_databuffer = 0;  
  CLEAR_EBIT;
  // the first timer interrupt will occur in the mid of the first half of the second start bit 
  TBCCR0 = TBR + BIT75;     // set first TBCCR1 IRQ to 75% of RC5 bitlength
  TBCCTL0 &= ~CCIFG;        // clear previous compare flag
  TBCCTL0 |= CCIE;          // CCR0 interrupt enabled, interrupt occurs when timer equals CCR0
  P1IE &= ~0x04;   // disable interrupt for P12 ( ReceiveIR )
  #endif
}
