/*
Copyright 2003/2004, Freie Universitaet Berlin. All rights reserved.
 
These sources were developed at the Freie Universitt Berlin, Computer
Systems and Telematics group.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:
 
- Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.
  
- Redistributions in binary form must reproduce the above copyright 
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.
 
- Neither the name of Freie Universitaet Berlin (FUB) nor the names of its
contributors may be used to endorse or promote products derived from
this software without specific prior written permission.

This software is provided by FUB and the contributors on an "as is"
basis, without any representations or warranties of any kind, express
or implied including, but not limited to, representations or
warranties of non-infringement, merchantability or fitness for a
particular purpose. In no event shall FUB or contributors be liable
for any direct, indirect, incidental, special, exemplary, or
consequential damages (including, but not limited to, procurement of
substitute goods or services; loss of use, data, or profits; or
business interruption) however caused and on any theory of liability,
whether in contract, strict liability, or tort (including negligence
or otherwise) arising in any way out of the use of this software, even
if advised of the possibility of such damage.

This implementation was developed by the CST group at the FUB.
Contributors: Marco Hutta

For documentation and questions please use the web site
http://www.scatterweb.net and the mailinglist
scatterweb@lists.spline.inf.fu-berlin.de (subscription via the Website).
Berlin, 2003/2004/2005
*/

#ifndef ScatterWeb_String
#define ScatterWeb_String

#include <stdarg.h>

/*
 * Function to convert int to String
 *  
 * @param writer    pointer of writer
 * @param number    value to convert
 * @param length    length to write
 * 
 * @return int      position in output out[pos]
 *  
 */
int String_writeUInt(fp_char_t writer, UINT16 number, UINT16 length);

/*
 * Function to convert long int to String
 *  
 * @param writer    pointer of writer
 * @param number    value to convert
 * @param length    length to write
 * 
 * @return int      position in output out[pos]
 *  
 */
int String_writeULongInt(fp_char_t writer, UINT32 number, UINT16 length);

/*
 * Function to convert Hexstring to byte
 * 
 * @param b         value to convert
 * @param cs        value to convert into
 * 
 * @return bool     True always
 *  
 */
bool String_fromByteToHex(UINT8 b, UINT8* cs);

/*
 * Function to convert byte to Hexstring
 *  
 * @param cs        value to convert
 * 
 * @return UINT8    value to convert into
 * 
 */
UINT8 String_fromHexToByte(const UINT8* cs);

/*
 * Function to write temperature
 *  
 * @param writer    pointer of writer ([+-]xxx.x)
 * @param temp      temperature to write
 * 
 * @return bool     True always
 *  
 */
bool String_writeTemperature(fp_char_t writer, temp_t temp);

/*
 * Function to abstract write
 *  
 * @param writer    pointer of writer
 * @param *fmt      kind of value   
 * @param argp      value to write
 * 
 * @return bool     True always
 *  
 */
bool String_abstractWrite(fp_char_t writer, const char *fmt, va_list argp);

/*
 * Function to write
 *  
 * @param dest      pointer of destination
 * @param *fmt      kind of value
 * @param ...       value to write
 * 
 * @return bool     True always
 *  
 */
bool String_write(UINT8* dest, const char *fmt, ...);

/*
 * Function to parse int
 * 
 * @param c         pointer of position
 * @param ptr       pointer of first position after the parsed int
 * 
 * @return UINT16   parsed int  
 *  
 */
UINT16 String_parseInt(const UINT8* c, UINT8** ptr);

/*
 * Function to parse long int
 * 
 * @param c           pointer of position
 * @param ptr         pointer of first position after the parsed long int
 * 
 * @return UINT16     parsed long int  
 *  
 */
UINT32 String_parseLongInt(const UINT8* c, UINT8** ptr);

#endif
