/*
Copyright 2003/2004, Freie Universitaet Berlin. All rights reserved.

These sources were developed at the Freie Universitt Berlin, Computer
Systems and Telematics group.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

- Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.
 
- Neither the name of Freie Universitaet Berlin (FUB) nor the names of its
contributors may be used to endorse or promote products derived from
this software without specific prior written permission.

This software is provided by FUB and the contributors on an "as is"
basis, without any representations or warranties of any kind, express
or implied including, but not limited to, representations or
warranties of non-infringement, merchantability or fitness for a
particular purpose. In no event shall FUB or contributors be liable
for any direct, indirect, incidental, special, exemplary, or
consequential damages (including, but not limited to, procurement of
substitute goods or services; loss of use, data, or profits; or
business interruption) however caused and on any theory of liability,
whether in contract, strict liability, or tort (including negligence
or otherwise) arising in any way out of the use of this software, even
if advised of the possibility of such damage.

This implementation was developed by the CST group at the FUB.
Contributors: Bjoern Lichtblau

For documentation and questions please use the web site
http://www.scatterweb.net and the mailinglist
scatterweb@lists.spline.inf.fu-berlin.de (subscription via the Website).
Berlin, 2003/2004/2005
*/

#include "ScatterWeb.System.h"

#define WDSTATE   (*((UINT16*)0x09fe))      ///< 2 bytes are reserved at main

/** The callbacks set via ::registerCallback are stored here. */
fp_vp callbacks[NUMBER_CALLBACKS];
// present possible flags
volatile UINT16 runModule;

UINT16 wd_startup;

UINT8* panic_dest;

const UINT8 reason_strings[7][10] = { "POR", "SWRESET", "REPROGRAM", "PANIC", 
                                             "WATCHDOG", "KEYV", "UNKNOWN" };

extern void Process_init();  ///< The only function in userapp that can be called from firmware (except callbacks).

// software wd
interrupt(TIMERA1_VECTOR) timera1_isr (void){
  UINT16 oldpc = * ( ((UINT16*)GET_FRAME_ADDR(timera1_isr)) + 1);
  UINT16 sp = READ_SP;
  System_panic("SW-WD(%x,%x)", oldpc, sp);
  TACCTL1 = 0x0000; // Ensure timer is off. clrs IE&IFG
  System_reset();
}

void wd_init(){
    // It was a POR/PUC.
    if(TACTL == 0x0000) wd_startup = STARTUP_POR;    
    // PUC only: look further.
    else {                                      
      if(FCTL3 & KEYV){ // PUC: Was it a flash key violation?
        FCTL3 = 0xA500; // Clear KEYV Bit in FCTL3, check if wanted or by accident.
        if(WDSTATE==WDS_SWRESET) wd_startup = STARTUP_SWRESET;
        else if(WDSTATE==WDS_PANIC) wd_startup = STARTUP_PANIC;
        else if(WDSTATE==WDS_REPROGRAM) wd_startup = STARTUP_REPROGRAM;
        else                      wd_startup = STARTUP_KEYV;
      }
      else if(IFG1 & WDTIFG){ // PUC: Was it the watchdog?
        IFG1 &= ~WDTIFG; // Clear WDTIFG Bit. 
        wd_startup = STARTUP_WATCHDOG;
        // WDSTATE could be further evaluated here, where occured the wdt expiry?
      }
      else { // PUC only but no watchdog? 
        wd_startup = STARTUP_UNKNOWN;
      }
    }
    WDSTATE = WDS_RUNNING;
}

void wd_stackCheck(){
  extern UINT16 __noinit_end;
  UINT16* p = & __noinit_end;
  if(p[0]!=0xABCD || p[1]!=0xABCD || p[2]!=0xABCD || p[3]!=0xABCD){
    if(Configuration.panicFlags & PANIC_ON_STACK_OVERFLOW) System_panic("Stack LOW");
    else System_reset();
  }
  Timers_add(1024, wd_stackCheck, 0xFFFF);    
}

/** Writes something to userappControl */
void panic_w(UINT8 c){
  if(panic_dest < (UINT8*)0x1080 || 
     panic_dest > (UINT8*)0x10FF) return;
  if(panic_dest == (UINT8*)0x10FF) c = 0x00;
  ///////////////////////////// disable all interrupts
  IFG1=0;                    // clear interrupt flag1 
  WDTCTL=0x5A80;             // stop WD 
                             // DCO(SMCLK) is 2,4576MHz, /6 = 409600 Hz 
  FCTL2=0xA5C5;              // select SMCLK for flash timing, divider 5+1 
  _DINT();                   // disable all interrupts to protect CPU 
                             // during programming from system crash 
  IE1=0x00;                  // disable all NMI-Interrupt sources
  /////////////////////////////  
  FCTL3 = 0xA500;              // Lock = 0 
  while(FCTL3 & 0x0001) nop(); // Wait for BUSY = 0, not needed unless run from RAM
  FCTL1 = 0xA540;              // WRT = 1 
  *panic_dest =  c;
  panic_dest++;    
  FCTL1 = 0xA500;              // WRT = 0 
  FCTL3 = 0xA510;              // Lock = 1 
  ///////////////////////////// Dotn need to reenable watchdog or ints as reset will follow
}

bool System_registerCallback(UINT8 type, fp_t fp) {
  callbacks[type] = fp;
  return True;
}

bool System_registerLib(UINT8 position, fp_t fp) {
  libregistry[position] = fp;
  return True;
}

bool System_reset() { 
  WDSTATE = WDS_SWRESET;
  FCTL3 = 0xDEAD;
  return True;
}

bool System_init() {
  wd_stackCheck();
  if((Configuration.panicFlags & PANIC_ON_BAD_RESET) && (wd_startup > STARTUP_PANIC)){
    System_panic("RESET[%s]", System_resetReason());
  }
  return True;
}

const UINT8* System_resetReason() {
  return reason_strings[wd_startup];
}

bool System_panicClear() {
  UINT8 ie1 = IE1;
  ///////////////////////////// disable all interrupts
  IFG1=0;                    // clear interrupt flag1 
  WDTCTL=0x5A80;             // stop WD 
                             // DCO(SMCLK) is 2,4576MHz, /6 = 409600 Hz 
  FCTL2=0xA5C5;              // select SMCLK for flash timing, divider 5+1 
  _DINT();                   // disable all interrupts to protect CPU 
                             // during programming from system crash 
  IE1=0x00;                  // disable all NMI-Interrupt sources
  /////////////////////////////
  FCTL3 = 0xA500;               // Lock = 0   
  while(FCTL3 & 0x0001) nop();  // Wait for BUSY = 0, not needed unless run from RAM
  FCTL1 = 0xA502;               // ERASE = 1 
  *((UINT16*)0x1080) = 0xFFFF;            // erase 2. Flash info segment 
  FCTL1 = 0xA500;               // ERASE = 0 automatically done?!
  FCTL3 = 0xA510;               // Lock = 1 
  ///////////////////////////// 
  HAL_startWatchdog();       // 
  _EINT();                   // 
  IFG1 = 0x00;               // 
  IE1= ie1;                  // 
  /////////////////////////////
  return True;
}

bool System_panic(const char *fmt, ...){
  extern volatile time_t currentTime; // interrupts are disabled so access direct
  va_list argp;
  btime_t btime;
  dint();
  IFG1 = 0x00;
  panic_dest = (UINT8*) 0x1080;
  while(*panic_dest != 0xFF && panic_dest < (UINT8*)0x10FF) panic_dest++;
  Time_convertSystemTime((time_t*)&currentTime, &btime);
  Time_writeTime(panic_w, &btime);  
  panic_w(' ');
  va_start(argp, fmt);
  String_abstractWrite(panic_w, fmt, argp);  
  va_end(argp);
  panic_w(0x00);
  WDSTATE = WDS_PANIC;
  _NOP();
  FCTL3 = 0xDEAD;
  return True;
}

UINT8* System_panicReason(UINT16 n){
  UINT8* p = (UINT8*)0x1080;
  UINT16 i;
  for(i=0; i<n; i++){
    while(*p != 0x00) p++;
    p++;
  }
  if(*p != 0xFF && p < (UINT8*)0x10FF) return p;
  else           return NULL;
}

bool System_wait(UINT32 w){
  volatile UINT32 i; 
  for(i=0;i<w;i++) nop(); 
  return True;
}

bool System_waitA(UINT16 i) {
  UINT16 tar = TAR;
  while(TAR - tar < i) HAL_startWatchdog();
  return True;
}

__attribute__((reserve(2))) int main(void) {
  Data_redOn();
  WDTCTL = WDT_ARST_1000;
  HAL_stopWatchdog();
  HAL_initPorts();
  wd_init();
  runModule = 0;
  HAL_initHardware();
  System_init();
  Configuration_print();
  volatile UINT16 i;
  for(i=0; i<NUMBER_CALLBACKS; i++) callbacks[i] = 0;  
  for(i=0; i<NUMBER_LIBS; i++) libregistry[i] = 0;  
  Threading_init();
  if((Configuration.panicFlags & PANIC_DISABLES_APP) && (System_panicReason(0) != NULL)){
    Comm_print("Info: Userapp disabled.\r\n");
    Terminal_enable();
  }
  else {
    Comm_print("Info: Userapp enabled.\r\n");
    Process_init();
  }
  Data_redOff();
  for(;;) {  
    HAL_startWatchdog(); // starts&resets the watchdog
    if(runModule & MF_SCOS) Threading_eventHandler();
    //// Radio tasks.
    if(runModule & MF_RADIO_RX) {
#ifdef waelchli_debug
    Comm_print("Radio receive, module %i\r\n", runModule);    
#endif
      Net_rxHandler(); 
    } 
    if(runModule & MF_RADIO_TX) {
#ifdef waelchli_debug
    Comm_print("Radio transceive, module %i\r\n", runModule);    
#endif
      Net_txHandler();
    }
    if(runModule & MF_TIMER) Timers_eventHandler();  

    if(runModule & MF_SERIAL_RX) { // Callback for serial line
      extern volatile UINT8* serial_line;
#ifdef debug_waelchli
      Comm_print("ES kommt was!!\r\n");
#endif
      if(serial_line != 0){
      	if(callbacks[C_SERIAL]) callbacks[C_SERIAL]((void*)serial_line);
      }
    }
    if(runModule & MF_CAM) {
      if (libregistry[LIB_CAM]) libregistry[LIB_CAM]();
    }
    if(runModule & MF_SENSORS) if(sensorFlags != 0x00) Data_sensorHandler();
    if(runModule & MF_RC5) {
      if (libregistry[LIB_RC5_REC]) libregistry[LIB_RC5_REC]();
    }
    dint(); 
    nop();
    if(runModule==0) { // enter lpm3 and enable GIE at once to not loose an interrupt
      HAL_stopWatchdog();
      eint();
      LPM1;
    } else eint();
  }  
}
