/*
Copyright 2003/2004, Freie Universitaet Berlin. All rights reserved.

These sources were developed at the Freie Universitt Berlin, Computer
Systems and Telematics group.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

- Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.
 
- Neither the name of Freie Universitaet Berlin (FUB) nor the names of its
contributors may be used to endorse or promote products derived from
this software without specific prior written permission.

This software is provided by FUB and the contributors on an "as is"
basis, without any representations or warranties of any kind, express
or implied including, but not limited to, representations or
warranties of non-infringement, merchantability or fitness for a
particular purpose. In no event shall FUB or contributors be liable
for any direct, indirect, incidental, special, exemplary, or
consequential damages (including, but not limited to, procurement of
substitute goods or services; loss of use, data, or profits; or
business interruption) however caused and on any theory of liability,
whether in contract, strict liability, or tort (including negligence
or otherwise) arising in any way out of the use of this software, even
if advised of the possibility of such damage.

This implementation was developed by the CST group at the FUB.
Contributors: Marco Hutta

For documentation and questions please use the web site
http://www.scatterweb.net and the mailinglist
scatterweb@lists.spline.inf.fu-berlin.de (subscription via the Website).
Berlin, 2003/2004
*/

#ifndef ScatterWeb_Terminal
#define ScatterWeb_Terminal

// Command with this flag access uart directly with prints.
#define SERIALONLY     (0x02)
// Use this MACRO to declare terminal command functions
#define COMMAND(x, flags)             void term_##x (const UINT8* str); \
                                      __attribute__((section(".commands"))) const command_t com_##x = { term_##x, #x, flags }; \
                                      void term_##x (const UINT8* str)

// Use this MACRO to declare terminal command functions without argument
#define COMMAND2(x, flags)           void term_##x (); \
                                     __attribute__((section(".commands"))) const command_t com_##x = { term_##x, #x, flags }; \
                                     void term_##x ()

// Terminal command function pointer type, argument is the command.
typedef void(*fp_term)(const UINT8*);
// Reply buffer. All commands (except SERIALONLY) build their reply here.
extern UINT8 term_reply[TERM_BUFSIZE];
// Struct for the command directory
typedef struct command_struct { 
  fp_term function;
  UINT8 code[3];
  UINT8 flags;
} command_t;

/*
 * Function to enable terminal
 * 
 * @return bool   True always  
 *  
 */
bool Terminal_enable();

/*
 * Function to disable terminal
 * 
 * @return bool   True always  
 *  
 */
bool Terminal_disable();

/*
 * Function to handle serial
 * 
 * @return bool   True always  
 *  
 */
void Terminal_serialHandler();

/*
 * Function to handle request
 * 
 * @return bool   True always  
 *  
 */
bool Terminal_requestHandler();

/*
 * Function to handle answer
 * 
 * @return bool   True always  
 *  
 */
bool Terminal_answerHandler();

/*
 * Function to handle data request
 * 
 * @return bool   True always  
 *  
 */
bool Terminal_requestDataHandler(UINT8* data);

#endif

